{% load i18n static %}

<div class="clearfix">
  <h3 class="table-title">{% trans "Deployment Logs" %}</h3>
</div>
<div class="reports logs">
    {% for report in reports %}
        <div title="{{report.level|capfirst}}" class="report-{{report.level}}">{{report.created}} &mdash; {{report.text | linebreaksbr | urlize}}</div>
    {% endfor %}
</div>
