{% extends 'horizon/common/_modal_form.html' %}
{% load i18n %}

{% block modal-body-right %}
    <h3>{% trans 'Description' %}:</h3>
    <p>{% trans 'Add new category to the application catalog.' %}</p>
    <p>
        <strong>{% trans 'Name' %}:</strong>
        {% trans "Provide desired name for a new category" %}
    </p>

{% endblock %}
