{% load i18n %}

<h3>{% trans "Description" %}:</h3>
<p><strong>{% trans "Name" %}: </strong>{% trans "Set up for identifying a package." %}</p>
<p><strong>{% trans "Tags" %}: </strong>{% trans "Used for identifying and filtering packages." %}</p>
<p><strong>{% trans "Public" %}: </strong>{% trans "Defines whether or not a package can be used by other tenants. (Applies to package dependencies)" %}</p>
<p><strong>{% trans "Active" %}: </strong>{% trans "Allows to hide a package from the catalog. (Applies to package dependencies)" %}</p>
<p><strong>{% trans "Description" %}: </strong>{% trans "Allows adding additional information about a package." %}</p>
