{% extends 'horizon/common/_modal_form.html' %}
{% load i18n %}

{% block modal-body-right %}
    <h3>{% trans "Description" %}:</h3>
    <p>
        <strong>{% trans "Environment Name" %}:</strong>
        {% trans "Choose a name for the environment" %}
    </p>
    <p>
        {% trans "An environment is a collection of applications that are meant to operate under similar conditions." %}
    </p>
{% endblock %}
