# The name of the panel to be added to HORIZON_CONFIG. Required.
PANEL = 'images'
# The name of the dashboard the PANEL associated with. Required.
PANEL_DASHBOARD = 'app-catalog'
# The name of the panel group the PANEL is associated with.
PANEL_GROUP = 'app-catalog_manage_group'

# Python panel class of the PANEL to be added.
ADD_PANEL = 'muranodashboard.images.panel.Images'
