{% extends 'base.html' %}
{% load i18n sizeformat %}

{% block title %}{% trans "Deployment Details" %}{% endblock %}

{% block page_header %}
  {% include "common/_detail_header.html" %}
{% endblock page_header %}

{% block main %}
<div class="row">
  <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">{{ tab_group.render }}</div>
</div>
{% endblock %}
