{% load i18n %}
{% load static %}
{% load compress %}

{% block css %}
  {% compress css %}
    <link rel="stylesheet" href="{% static 'muranodashboard/css/deployments.css' %}">
  {% endcompress %}
{% endblock %}

{% block main %}
<table class="{% block table_css_classes %}table deployment_history_cell{% endblock %}">
<thead></thead>
<tbody>
{% for name, type in services.items %}
<tr>
    <td>{{name}} &mdash; {{type | linebreaksbr}}</td>
</tr>
{% endfor %}
</tbody>
</table>
{% endblock %}