{% extends 'horizon/common/_modal_form.html' %}
{% load i18n %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:app-catalog:packages:modify' app_id %}{% endblock %}

{% block modal-header %}{% trans "Modify Package" %}{% endblock %}
{% block modal_id %}modify_package_modal{% endblock %}

{% block modal-body-right %}
    {% include 'packages/_package_params.html' %}
    {% if type != 'Library' %}
      <p><strong>{% trans "Categories" %}. </strong>{% trans "Select one or more categories for a package." %}</p>
    {% endif %}
{% endblock %}

{% block modal-footer %}
    <a href="{% url 'horizon:app-catalog:packages:index' %}" class="btn btn-default cancel">{% trans "Cancel" %}</a>
    <input type='submit' class='btn btn-primary' value='{% trans "Update" %}' />
{% endblock %}
