{% extends 'base.html' %}
{% load i18n %}
{% load static %}
{% load compress %}
{% block title %}{% trans "Environments" %}{% endblock %}

{% block main %}
  {{ table.render }}
{% endblock %}

{% block css %}
  {{ block.super }}
  {% compress css %}
    <link href="{% static 'muranodashboard/css/catalog.css' %}" type='text/css' media='screen' rel='stylesheet' />
  {% endcompress %}
{% endblock %}
