{% load i18n %}

<div class="status row detail app_overview">
 <div class="app-meta">
     <div class="app-metadata">
       <div class="app-description">{% if app.description %}{{ app.description }}{% endif %}</div>
       <ul>
        {% if app.author %}<li><strong>{% trans "Author" %}:</strong> {{ app.author }}</li>{% endif %}
        {% if app.tags %}<li><strong>{% trans "Tags" %}:</strong> {{ app.tags|join:", " }}</li>{% endif %}
        {% if app.categories %}<li><strong>{% trans "Categories" %}:</strong> {{ app.categories|join:", " }}</li>{% endif %}
       </ul>
     </div>

   {% if app.supplier.Name %}
   <div class="app-supplier clearfix">
     {% if app.supplier.Logo %}
     <div class="app-icon">
       <img src="{% url 'horizon:app-catalog:catalog:supplier_images' app.id %}" />
     </div>
     {% endif %}
     <div class="app-supplier-data">
       <ul>
        {% if app.supplier.Name %}<li><strong>{{ app.supplier.Name }}</strong></li>{% endif %}
        {% if app.supplier.CompanyUrl.Link %}
          {% if app.supplier.CompanyUrl.Text %}
            <li><a href="{{ app.supplier.CompanyUrl.Link }}">{{ app.supplier.CompanyUrl.Text }}</a></li>
          {% else %}
            <li><a href="{{ app.supplier.CompanyUrl.Link }}">{{ app.supplier.CompanyUrl.Link }}</a></li>
          {% endif %}
        {% endif %}
        {% if app.supplier.Summary %}<li>{{ app.supplier.Summary }}</li>{% endif %}
        {% if app.supplier.Description %}<li>{{ app.supplier.Description }}</li>{% endif %}
       </ul>
     </div>
   </div>
   {% endif %}
</div>
</div>
