{% extends 'horizon/common/_modal_form.html' %}
{% load i18n %}

{% block modal-body-right %}
    <h3>{% trans "Description" %}:</h3>
    <p>{% trans "Mark an image with Murano specific metadata to be added to the selected image." %}</p>
    <p>
        <strong>{% trans "Image" %}:</strong>
        {% trans "Select an image registered in Glance Image Services." %}
    </p>
    <p>
        <strong>{% trans "Image Title" %}:</strong>
        {% trans "Create a title for an image." %}
    </p>
    <p>
        <strong>{% trans "Image Type" %}:</strong>
        {% trans "Select an image type supported by Murano. Choose 'Custom type' to enter type manually." %}
    </p>
    <p>
        <strong>{% trans "Custom Type" %}:</strong>
        {% trans "Enter an image type supported by Murano." %}
    </p>

{% endblock %}
