{% load i18n %}


<div class="status row detail app_requirements">
        {% if application.requirements %}
            <ul>{{ application.requirements|unordered_list }}</ul>
        {% else %}
            <p>{% trans 'No requirements' %}</p>
        {% endif %}
</div>
