{% extends 'base.html' %}
{% load i18n sizeformat %}
{% load static %}
{% load compress %}
{% block title %}{% trans "Application Details" %}: {{ app.name }}{% endblock %}

{% block css %}
  {{ block.super }}
  {% compress css %}
    <link rel="stylesheet" href="{% static 'muranodashboard/css/catalog.css' %}">
    {% if HORIZON_CONFIG.legacy_static_settings %}
      <link rel="stylesheet" href="{% static 'horizon/lib/font-awesome/css/font-awesome.min.css' %}">
    {% else %}
      <link rel="stylesheet" href="{% static 'horizon/lib/font_awesome/css/font-awesome.min.css' %}">
    {% endif %}
  {% endcompress %}
{% endblock %}

{% block page_header %}
  {% include "common/_detail_header.html" %}
{% endblock %}

{% block main %}
<div class="table_wrapper">
  <div class="row">
    <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
      <div class="btn-group pull-right">
        {% with class='btn btn-default btn-sm btn-add ajax-modal' %}
            {% include 'catalog/add_app.html' %}
        {% endwith %}
        {% with class='btn btn-default btn-sm btn-add ajax-modal' %}
            {% include 'catalog/quick_deploy.html' %}
        {% endwith %}
      </div>
      <div class="clearfix">
        <div class="app-icon pull-left">
          <img src="{% url 'horizon:app-catalog:catalog:images' app.id %}" />
        </div>
      </div>
      {{ tab_group.render }}
    </div>
  </div>
</div>
{% endblock %}
