{% load i18n %}

{% block main %}
<div class="row">
  <div class="col-sm-12">
    {% include "services/_data_table.html" %}
    <hr>
    <div id="components">
      {{ table.render }}
    </div>
  </div>
</div>
{% endblock %}
