{% load i18n %}
{% load static %}

<div class="col-xs-12 col-sm-6 col-md-6 col-lg-4 col">
    <div class="well app">
        <div class="app-description">
            <div class="col-xs-2 col-sm-4 col-md-4 col-lg-4 app-icon">
                <a href="{% url 'horizon:app-catalog:catalog:application_details' app.id %}"
                   class="btn-link">
                    <img src="{% url 'horizon:app-catalog:catalog:images' app.id %}"/>
                </a>
            </div>
            <div class="col-xs-10 col-sm-8 col-md-8 col-lg-8 description">
                <h4>{{ app.name }}</h4>

                <p>{{ app.description|striptags|truncatechars:130 }}</p>

                <div class="app-details-link"><a
                        href="{% url 'horizon:app-catalog:catalog:application_details' app.id %}"
                        class="btn-link">{% trans "Details" %} &raquo;</a></div>
            </div>

        </div>
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12 app-actions-container">
            <div class="app-actions">
                <div class="btn-group">
                    {% with class='btn btn-default btn-sm  ajax-modal' %}
                        {% include 'catalog/add_app.html' %}
                    {% endwith %}
                    {% with class='btn btn-default btn-sm ajax-modal' %}
                        {% include 'catalog/quick_deploy.html' %}
                    {% endwith %}
                </div>
            </div>
        </div>
        {% if app.owner_id != tenant_id %}
            <img class="ribbon"
                 src="{% static 'muranodashboard/images/shared.png' %}">
        {% endif %}
        <div class="clearfix"></div>
    </div>
</div>
